/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file ImageObject.h
 *    Declaration of ImageObject.
 *
 * @author Tim Langner
 * @date   2011/01/06
 */

#ifndef _MIRA_IMAGEOBJECT_H_
#define _MIRA_IMAGEOBJECT_H_

#include <image/Img.h>
#include <image/Colormap.h>

#include <visualization/3d/VisualizationObject.h>

#ifndef Q_MOC_RUN
#include <OGRE/OgreMaterial.h>
#include <OGRE/OgreTexture.h>
#endif

namespace Ogre {
	class ManualObject;
}

namespace mira {

//////////////////////////////////////////////////////////////////////////////

class MIRA_GUI_VISUALIZATION_EXPORT ImageObject : public VisualizationObject
{
public:
	ImageObject(bool mirrorY, Ogre::SceneManager* sceneManager, Ogre::SceneNode* parent = NULL);
	virtual ~ImageObject();

	void setAlpha(float alpha);
	void setImage(const Img<>& img, double smin, double smax, const ContinuousColormap& colormap, bool enableMipmapping=false);
	void setImage(const Img<>& img, bool enableMipmapping=false);

	/// Returns a pointer to the internal manual object representing the image plane
	Ogre::ManualObject* getPlane() { return mPlane; }

	/// Returns internal material
	Ogre::MaterialPtr getMaterial() { return mMaterial; }

	Ogre::TexturePtr getTexture() { return mTexture; }

	Ogre::TextureUnitState* getTextureState();



protected:
	Ogre::ManualObject* mPlane;
	Ogre::MaterialPtr mMaterial;
	Ogre::TexturePtr mTexture;
	Ogre::PixelFormat mTexturePxf;
};

//////////////////////////////////////////////////////////////////////////////

}

#endif
